#################
#Install packages
#################
#Run  the commands below to make sure you have the latest package version

#install.packages("seqinr")
#install.packages("ape")
#install.packages("stats")

#Load the packages

library(seqinr)
library(stats)
library(ape)


##############
#Load the data
##############

files.human <- list.files("PDZLigands/human", pattern="*.fa", full.names=TRUE)
files.worm <- list.files("PDZLigands/worm", pattern="*.fa", full.names=TRUE)


#########################
#Get the PDZ domain names
#########################

#Remove the extension (".pep.fa") by splitting the string according to the delimiter ".pep"
nm <- c()
for (f in files.human){
  nm <- c(nm, strsplit(f, split=".pep", fixed=T )[[1]][1])
}
#Remove the path ("PDZLigands/human/")
PDZ.human <- c()
for (x in nm){
  PDZ.human <- c(PDZ.human, strsplit(x, split="/", fixed=T )[[1]][3])
}

#Do the same for the worm data. Use a more concise approach with the lapply() function
nm <- lapply(files.worm, function(x){strsplit(x, split=".pep", fixed=T )[[1]][1] })
PDZ.worm <- lapply(nm, function(x){strsplit(x, split="/", fixed=T )[[1]][3] })
PDZ.worm <- unlist(PDZ.worm)   #This is because the output of lapply() is a list and not a vector

#This would be an even more concise version for loading the PDZ names in Worm
#PDZ.worm <- unlist(lapply(files.worm, function(x){strsplit(  strsplit(x, ".pep")[[1]][1], "/", fixed=T )[[1]][3] }))

PDZ.all <- c(PDZ.human, PDZ.worm)
le <- length(PDZ.all)

PDZ.org <- c(rep("H", length(PDZ.human)), rep("W", length(PDZ.worm)))
names(PDZ.org) <- PDZ.all


####################
#Get the PDZ classes
####################
PDZ.class <- read.table("PDZclass.txt", skip=1)
class2 <- as.numeric(PDZ.class[,2])
names(class2) <- PDZ.all
class16 <- as.character(PDZ.class[,3])
names(class16) <- PDZ.all


#########################
#Load the background freq 
#########################
#This is only useful for the clustering in part 7
b <- read.table("phageLibraryNNKTheoreticalCodonBias.txt")
bg.fr <- b[,2]


########################################################
#Get the sequence of the peptide ligands for each domain
########################################################
loadFile <- function(x){
  f <- read.fasta(file=x, as.string = TRUE, seqtype = "AA")
  nm <- names(f)
  seq <- unlist(lapply(nm, function(x){f[[x]][1]}))
  return(seq)
}

seq.human <- lapply(files.human, loadFile)
seq.worm <- lapply(files.worm, loadFile)
seq.all <- c(seq.human, seq.worm)
names(seq.all) <- PDZ.all


#######################################
#Load the PDZ domain sequence alignment
#######################################
#f <- read.fasta(file="PDZ_SMART_MUSCLE_sub.fa", as.string = TRUE, seqtype = "AA")
f <- read.fasta(file="PDZ_SMART_CLUSTAL_sub.fa", as.string = TRUE, seqtype = "AA")
#f <- read.fasta(file="PDZ_phage_MUSCLE.fa", as.string = TRUE, seqtype = "AA")

PDZ.seq <- unlist(lapply(PDZ.all, function(x){f[[x]][1]})) # Extract the sequence of each PDZ domain
bs <- which(unlist(strsplit(f[[1]][1], ""))=="B")  # Find the position of the binding site

bs.seq <- unlist(lapply(PDZ.seq, function(x){ paste(unlist(strsplit(x, ""))[bs], sep="", collapse="") }))
names(bs.seq) <- PDZ.all



####################
#Create PWM matrices - point 2c
####################

#This is useful to assign a integer to each amino acid, including gaps ('X' or '-')
Naa <- 20 #Number of amino acids
gap <- 1/Naa #Contribution of gaps in the PWM
map <- c(1:(Naa+1), Naa+1)
names(map) <- c("A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y", "X", "-")

#This is the map for grouping amino acids based on their physico-chemical properties (only useful for part 7)
reduced_map <- c(4, 4, 3, 3, 4, 5, 2, 4, 2, 4, 4, 1, 4, 1, 2, 1, 1, 4, 4, 4)
names(reduced_map) <- c("A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y")

#Choose between the two options for the two Figures to be reproduced (first Fig <- 7 and then Fig <- 2)
Fig <- 7
#Fig <- 2


pwm.all <- list()
#Go through the set of ligands for each PDZ domain
ct <- 1
for(s in seq.all){
  x <- unlist(s)
  
  #Build a 20x10 matrix counting how often each amino acid occurs at each position
  #Use the map vector to know the position (matrix row) corresponding to each amino acid. 
  #For 'X' or '-', add 'gap=1/20' in the whole column (non-specific).
  
  m <- matrix(0, nrow=Naa, ncol=nchar(x[1]))
  for(p in x){
    #...
   
  }
  
  if(Fig==2){
    #Only when doing the clustering (part 7), add the codon bias.
    #...
    
    
    #Only when doing the clustering (part 7), group the lines corresponding to similar amino acids (based on reduced_map)
    #...
   
  }
  
  #Normalize each column
  #...

  
  pwm.all[[ct]] <- m
  ct <- ct+1
}

####################################
#Compute the similarity between PWMs - point 2d
####################################

compPWMSim <- function(x,y){
  
  #Compute the similarity between two pwm matrices as defined in the article
  #...
  
  
}

#Build a (le x le) matrix with similarity values between every pair of PWMs
sim.pwm <- matrix(nrow=le, ncol=le)
for(i in 1:(le-1)){
  sim.pwm[i,i] <- 1
  for(j in (i+1):le){
    sim.pwm[i,j] <- compPWMSim(pwm.all[[i]], pwm.all[[j]]) 
    sim.pwm[j,i] <- sim.pwm[i,j]
  }
}
sim.pwm[le,le] <- 1
rownames(sim.pwm) <- PDZ.all
colnames(sim.pwm) <- PDZ.all


############################################
#Compute the distances in the sequence space - point 3
############################################

compSeqSim <- function(x,y){
  
  #Compute the precentage of identity between two PDZ domain binding sites (string x and y).
  #Do not consider positions where one of the sequences has a gap.
  #...
  
}

#Build a le x le matrix with similarity values between every pair of PDZ binding site (bs.seq)
sim.seq <- matrix(nrow=le, ncol=le)
#...

#################################
#Plot the two kinds of similarity as in Fig 7 - point 4
#################################

#Build a le x le matrix  with 1 if the two PDZ domains are in the same class, 0 if not, based on the class16.
sim.class <- matrix(nrow=le, ncol=le)
#...


#Plot the comparison of similarity values, with different colors whether the pairs of PDZ domains are in the same class (ind1) or not (ind2)
ind1 <- which(sim.class==1)
ind2 <- which(sim.class==0)
if(Fig==7){
  pdf("output/Fig7_compSim.pdf")
  plot(sim.seq[ind2], sim.pwm[ind2], col="blue", xlim=c(0,1), ylim=c(0.4,1), pch=0, cex=0.5) #This is similar to Figure 2
  points(sim.seq[ind1], sim.pwm[ind1], col="red") 
  dev.off()
}

##################
#Do the clustering - point 7
##################                     

#Go back to the loop to build the PWM

#Include the codon bias (basically divide the counts observed for each amino acid by the bias in bg.fr)
#....

#Include the merging of similar amino acids in compFreq, based on the 'reduced_map' vector (basically merge the rows corresponding to the same values in reduced_map). 
#This will create PWM of size 5x10, instead of 20x10
#...

#Define a distance object as 1-sim.pwm and the as.dist function
if(Fig==2){ 
  dis <- 1-sim.pwm
  rownames(dis) <- unlist(lapply(1:le, function(x){paste(class2[x], class16[x], PDZ.all[x], sep="  ")})) #This will be useful to show the classes on the dendrogram
  d <- as.dist(dis)

  #Do the hierarchical clustering
  hc <- hclust(d, method = "ward.D")

  #Plot the data with plot(as.dendrogram(hc), horiz=T, axes=F). Use par(cex=0.5, mar=c(5, 2, 2, 8)) to define the margin and text size
  #...

}

